@@ -49,7 +49,6 @@
 #include <string.h>
 
 #ifdef OS_UNIX
-#include <paths.h>
 #include <pwd.h>
 #include <unistd.h>
 #endif
@@ -63,9 +62,9 @@ char *
 p11_path_base (const char *path)
 {
 #ifdef OS_WIN32
-	static const char *delims = "/\\";
+	const char *delims = "/\\";
 #else
-	static const char *delims = "/";
+	const char *delims = "/";
 #endif
 
 	const char *end;
@@ -92,25 +91,60 @@ p11_path_base (const char *path)
 	return strndup (beg, end - beg);
 }
 
+static inline bool
+is_path_component_or_null (char ch)
+{
+	return (ch == '\0' || ch == '/'
+#ifdef OS_WIN32
+			|| ch == '\\'
+#endif
+		);
+}
+
 static char *
 expand_homedir (const char *remainder)
 {
 	const char *env;
 
+	if (getauxval (AT_SECURE)) {
+		errno = EPERM;
+		return NULL;
+	}
+
+	while (remainder[0] && is_path_component_or_null (remainder[0]))
+		remainder++;
+	if (remainder[0] == '\0')
+		remainder = NULL;
+
+	/* Expand $XDG_CONFIG_HOME */
+	if (remainder != NULL &&
+	    strncmp (remainder, ".config", 7) == 0 &&
+	    is_path_component_or_null (remainder[7])) {
+		env = getenv ("XDG_CONFIG_HOME");
+		if (env && env[0])
+			return p11_path_build (env, remainder + 8, NULL);
+	}
+
 	env = getenv ("HOME");
 	if (env && env[0]) {
 		return p11_path_build (env, remainder, NULL);
 
 	} else {
 #ifdef OS_UNIX
+		char buf[1024];
+		struct passwd pws;
 		struct passwd *pwd;
 		int error = 0;
+		int ret;
 
-		pwd = getpwuid (getuid ());
-		if (!pwd) {
+		ret = getpwuid_r (getuid (), &pws, buf, sizeof (buf), &pwd);
+		if (ret == 0 && !pwd) {
+			ret = -1;
+			errno = ESRCH;
+		}
+		if (ret < 0) {
 			error = errno;
-			p11_message ("couldn't lookup home directory for user %d: %s",
-				     getuid (), strerror (errno));
+			p11_message_err (errno, "couldn't lookup home directory for user %d", getuid ());
 			errno = error;
 			return NULL;
 		}
@@ -132,48 +166,6 @@ expand_homedir (const char *remainder)
 	}
 }
 
-static char *
-expand_tempdir (const char *remainder)
-{
-	const char *env;
-
-	env = getenv ("TEMP");
-	if (env && env[0]) {
-		return p11_path_build (env, remainder, NULL);
-
-	} else {
-#ifdef OS_UNIX
-#ifdef _PATH_TMP
-		return p11_path_build (_PATH_TMP, remainder, NULL);
-#else
-		return p11_path_build ("/tmp", remainder, NULL);
-#endif
-
-#else /* OS_WIN32 */
-		char directory[MAX_PATH + 1];
-
-		if (!GetTempPathA (MAX_PATH + 1, directory)) {
-			p11_message ("couldn't lookup temp directory");
-			errno = ENOTDIR;
-			return NULL;
-		}
-
-		return p11_path_build (directory, remainder, NULL);
-
-#endif /* OS_WIN32 */
-	}
-}
-
-static bool
-is_path_component_or_null (char ch)
-{
-	return (ch == '0' || ch == '/'
-#ifdef OS_WIN32
-			|| ch == '\\'
-#endif
-		);
-}
-
 char *
 p11_path_expand (const char *path)
 {
@@ -181,15 +173,7 @@ p11_path_expand (const char *path)
 
 	if (strncmp (path, "~", 1) == 0 &&
 	    is_path_component_or_null (path[1])) {
-		return expand_homedir (path + 2);
-
-	} else if (strncmp (path, "$HOME", 5) == 0 &&
-	    is_path_component_or_null (path[5])) {
-		return expand_homedir (path + 6);
-
-	} else if (strncmp (path, "$TEMP", 5) == 0 &&
-	    is_path_component_or_null (path[5])) {
-		return expand_tempdir (path + 6);
+		return expand_homedir (path + 1);
 
 	} else {
 		return strdup (path);
@@ -213,15 +197,16 @@ p11_path_build (const char *path,
                 ...)
 {
 #ifdef OS_WIN32
-	static const char delim = '\\';
+	const char delim = '\\';
 #else
-	static const char delim = '/';
+	const char delim = '/';
 #endif
 	const char *first = path;
 	char *built;
 	size_t len;
 	size_t at;
 	size_t num;
+	size_t until;
 	va_list va;
 
 	return_val_if_fail (path != NULL, NULL);
@@ -241,14 +226,28 @@ p11_path_build (const char *path,
 	path = first;
 	va_start (va, path);
 	while (path != NULL) {
-		if (at != 0 && built[at - 1] != delim && path[0] != delim)
-			built[at++] = delim;
 		num = strlen (path);
+
+		/* Trim end of the path */
+		until = (at > 0) ? 0 : 1;
+		while (num > until && is_path_component_or_null (path[num - 1]))
+			num--;
+
+		if (at != 0) {
+			if (num == 0)
+				continue;
+			built[at++] = delim;
+		}
+
 		assert (at + num < len);
 		memcpy (built + at, path, num);
-
 		at += num;
+
 		path = va_arg (va, const char *);
+
+		/* Trim beginning of path */
+		while (path && path[0] && is_path_component_or_null (path[0]))
+			path++;
 	}
 	va_end (va);
 
@@ -256,3 +255,71 @@ p11_path_build (const char *path,
 	built[at] = '\0';
 	return built;
 }
+
+char *
+p11_path_parent (const char *path)
+{
+	const char *e;
+	char *parent;
+	bool had = false;
+
+	return_val_if_fail (path != NULL, NULL);
+
+	/* Find the end of the last component */
+	e = path + strlen (path);
+	while (e != path && is_path_component_or_null (*e))
+		e--;
+
+	/* Find the beginning of the last component */
+	while (e != path && !is_path_component_or_null (*e)) {
+		had = true;
+		e--;
+	}
+
+	/* Find the end of the last component */
+	while (e != path && is_path_component_or_null (*e))
+		e--;
+
+	if (e == path) {
+		if (!had)
+			return NULL;
+		parent = strdup ("/");
+	} else {
+		parent = strndup (path, (e - path) + 1);
+	}
+
+	return_val_if_fail (parent != NULL, NULL);
+	return parent;
+}
+
+bool
+p11_path_prefix (const char *string,
+                 const char *prefix)
+{
+	int a, b;
+
+	return_val_if_fail (string != NULL, false);
+	return_val_if_fail (prefix != NULL, false);
+
+	a = strlen (string);
+	b = strlen (prefix);
+
+	return a > b &&
+	       strncmp (string, prefix, b) == 0 &&
+	       is_path_component_or_null (string[b]);
+}
+
+void
+p11_path_canon (char *name)
+{
+	static const char *VALID =
+		"abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789.-_";
+	int i;
+
+	return_if_fail (name != NULL);
+
+	for (i = 0; name[i] != '\0'; i++) {
+		if (strchr (VALID, name[i]) == NULL)
+			name[i] = '_';
+	}
+}
