@@ -91,7 +91,10 @@ main (int argc,
 {
 	p11_test (test_strndup, "/compat/strndup");
 #ifdef OS_UNIX
-	p11_test (test_getauxval, "/compat/getauxval");
+	/* Don't run this test when under fakeroot */
+	if (!getenv ("FAKED_MODE")) {
+		p11_test (test_getauxval, "/compat/getauxval");
+	}
 #endif
 	return p11_test_run (argc, argv);
 }
