@@ -61,7 +61,7 @@ p11_session_new (p11_token *token)
 	session->builder = p11_builder_new (P11_BUILDER_FLAG_NONE);
 	return_val_if_fail (session->builder, NULL);
 
-	session->index = p11_index_new (p11_builder_build, NULL,
+	session->index = p11_index_new (p11_builder_build, NULL, NULL,
 	                                p11_builder_changed,
 	                                session->builder);
 	return_val_if_fail (session->index != NULL, NULL);
